"""Test_main.py: Test GrabLib."""

import pytest
import requests

import grablib

pytestmark = pytest.mark.asyncio


class TestGrabLib:
    """Test the GrabLib module."""
    fake_url = "https://fake.url/filename.ext"

    async def test_get(self, mocker):
        """Test retrieval of URL."""
        mocker.patch("grablib.main.requests.get")
        fake_file_data = b"\x00\x00\x00\x00"
        response = mocker.MagicMock()
        response.status_code = 200
        response.content = fake_file_data
        grablib.main.requests.get.return_value = response
        with grablib.GrabLib() as grabber:
            actual = await grabber.get(self.fake_url)
            expected = {
                "url": self.fake_url,
                "sha256": (
                    "df3f619804a92fdb4057192dc43dd748"
                    "ea778adc52bc498ce80524c014b81119"
                ),
                "data": fake_file_data,
            }
        grablib.main.requests.get.assert_called_with(
            self.fake_url
        )
        assert actual == expected

    async def test_get_error(self, mocker):
        """Test error handling with URL retrieval."""
        mocker.patch("grablib.main.requests.get")
        error_msg = b"404 Error: File not found."
        response = mocker.MagicMock()
        response.status_code = 404
        response.content = error_msg
        grablib.main.requests.get.return_value = response
        with grablib.GrabLib() as grabber:
            actual = await grabber.get(self.fake_url)
            expected = {
                "url": self.fake_url,
                "sha256": "0" * 64,
                "data": b"Error: 404",
            }
        assert actual == expected

    async def test_get_exceptions(self, mocker):
        """Test exception handling with URL retrieval."""
        mocker.patch("grablib.main.requests.get")
        grablib.main.requests.get.side_effect = [
            requests.exceptions.ConnectionError,
            requests.exceptions.ProxyError,
            requests.exceptions.SSLError,
        ]
        response_messages = [
            b"Exception: ConnectionError",
            b"Exception: ProxyError",
            b"Exception: SSLError",
        ]
        with grablib.GrabLib() as grabber:
            for response_message in response_messages:
                actual = await grabber.get(self.fake_url)
                expected = {
                    "url": self.fake_url,
                    "sha256": "0" * 64,
                    "data": response_message,
                }
                assert actual == expected



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="redeye-grablib",
    version="0.0.2",
    author="CMSteffen",
    author_email="cmsteffen@haxys.net",
    description="A library for retrieving files from HTTP servers.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/RedEyeApp/grablib",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: CC0 1.0 Universal (CC0 1.0) Public Domain Dedication",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
    install_requires=[
        "requests",
    ],
)



"""Main.py: Home of the GrabLib class."""

import asyncio
import hashlib

import requests


class GrabLib:
    """Image Grabber: Downloads files from HTTP servers."""

    def __init__(self):
        """Initialize GrabLib."""

    def __enter__(self):
        """Elevate the Self."""
        return self

    def __exit__(self, exc_type, exc_value, exc_traceback):
        """Eliminate the Self."""

    async def get(self, file_url):
        """Retrieve the provided URL."""
        try:
            loop = asyncio.get_event_loop()
            response = await loop.run_in_executor(
                None, requests.get, file_url
            )
            data = response.content
            if response.status_code != 200:
                file_sha256 = "0" * 64
                data = f"Error: {response.status_code}".encode()
            else:
                sha256 = hashlib.sha256()
                sha256.update(response.content)
                file_sha256 = sha256.hexdigest()
        except Exception as ex:
            file_sha256 = "0" * 64
            data = f"Exception: {type(ex).__name__}".encode()
        return {"url": file_url, "sha256": file_sha256, "data": data}
